@@ -30,7 +30,6 @@
 #ifndef NO_STDINT_H
 #include <stdint.h>
 #endif
-#include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h> /* for snprintf() */
 
@@ -84,9 +83,18 @@
 // Win32 needs different close function for sockets because of
 // it's broken design.
 #ifdef __WIN32
-#define _CLOSE(x) closesocket((x))
+#define __CLOSE(x) closesocket((x))
 #else
-#define _CLOSE(x) { int __olderr = errno; close((x)); errno = __olderr; }
+#define __CLOSE(x) { int __olderr = errno; close((x)); errno = __olderr; }
+#endif
+#if   MUROAR_HDLTYPE == MUROAR_HDLTYPE_POSIX_SYSIO
+#define _CLOSE(x) __CLOSE((x))
+#elif MUROAR_HDLTYPE == MUROAR_HDLTYPE_POSIX_STDIO
+#define _CLOSE(x) { int __olderr = errno; fclose((x)); errno = __olderr; }
+#elif MUROAR_HDLTYPE == MUROAR_HDLTYPE_WIN32_SOCKET
+#define _CLOSE(x) __CLOSE((x))
+#else
+#error Unsupported value for MUROAR_HDLTYPE
 #endif
 
 // Set errno in case we have it
@@ -111,7 +119,7 @@ static void muroar_init_win32 (void) {
 #endif
 
 // Open Socket to server.
-static int muroar_open_socket (char * server) {
+static int muroar_open_socket_bsd (const char * server) {
  struct hostent     * he;
  struct sockaddr_in   in;
 #ifdef HAVE_AF_UNIX
@@ -141,7 +149,7 @@ static int muroar_open_socket (char * server) {
   strncpy(un.sun_path, server, sizeof(un.sun_path) - 1);
 
   if ( connect(fh, (struct sockaddr *)&un, sizeof(struct sockaddr_un)) == -1 ) {
-   _CLOSE(fh);
+   __CLOSE(fh);
    return -1;
   }
 
@@ -211,21 +219,44 @@ static int muroar_open_socket (char * server) {
  in.sin_port   = htons(MUROAR_PORT);
 
  if ( connect(fh, (const struct sockaddr *)&in, sizeof(in)) == -1 ) {
-  _CLOSE(fh);
+  __CLOSE(fh);
   return -1;
  }
 
  return fh;
 }
 
+static muroar_t muroar_open_socket (const char * server) {
+ int fh = muroar_open_socket_bsd(server);
+ muroar_t ret;
+
+ if ( fh == -1 )
+  return MUROAR_HANDLE_INVALID;
+
+#if MUROAR_HDLTYPE == MUROAR_HDLTYPE_POSIX_SYSIO || MUROAR_HDLTYPE == MUROAR_HDLTYPE_WIN32_SOCKET
+ ret = fh;
+#elif MUROAR_HDLTYPE == MUROAR_HDLTYPE_POSIX_STDIO
+ ret = fdopen(fh, "a+");
+#else
+#error Unsupported value for MUROAR_HDLTYPE
+#endif
+
+ if ( ret == MUROAR_HANDLE_INVALID ) {
+  __CLOSE(fh);
+  return MUROAR_HANDLE_INVALID;
+ }
+
+ return ret;
+}
+
 // Open Control connection
-int muroar_connect(char * server, char * name) {
+muroar_t muroar_connect(const char * server, const char * name) {
  char   useraddr[80] = "/invalid";
  char * addr[] = {useraddr, MUROAR_GSOCK, MUROAR_HOST, NULL};
  unsigned char buf[MUROAR_IOBUF];
  uint16_t tmpu16;
  uint16_t pid;
- int fh = -1;
+ muroar_t fh = MUROAR_HANDLE_INVALID;
  int i;
 #if !defined(__WIN32)
  ssize_t len;
@@ -249,21 +280,21 @@ int muroar_connect(char * server, char * name) {
 
  // Connect to server:
  if ( server != NULL ) {
-  if ( (fh = muroar_open_socket(server)) == -1 )
-   return -1;
+  if ( (fh = muroar_open_socket(server)) == MUROAR_HANDLE_INVALID )
+   return MUROAR_HANDLE_INVALID;
  } else {
   // build string for ~/.roar
   snprintf(useraddr, sizeof(useraddr), "/%s/.roar", getenv("HOME"));
   useraddr[sizeof(useraddr)-1] = 0;
 
   // go thru list of possible defaults:
-  for (i = 0; fh == -1 && addr[i] != NULL; i++) {
+  for (i = 0; fh == MUROAR_HANDLE_INVALID && addr[i] != NULL; i++) {
    fh = muroar_open_socket(addr[i]);
   }
 
-  if ( fh == -1 ) {
+  if ( fh == MUROAR_HANDLE_INVALID ) {
    _SET_ERRNO(ENOENT);
-   return -1;
+   return MUROAR_HANDLE_INVALID;
   }
  }
 
@@ -282,8 +313,9 @@ int muroar_connect(char * server, char * name) {
  // check if we have space for 5 bytes + length of name + tailing \0
  // in the buffer.
  if ( tmpu16 >= MUROAR_IOBUF ) {
+  _CLOSE(fh);
   _SET_ERRNO(EINVAL);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  buf[8] = (tmpu16 & 0xFF00) >> 8;
@@ -291,7 +323,7 @@ int muroar_connect(char * server, char * name) {
 
  if ( muroar_write(fh, buf, 10) != 10 ) {
   _CLOSE(fh);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  buf[0] = 1;
@@ -306,18 +338,18 @@ int muroar_connect(char * server, char * name) {
 
  if ( muroar_write(fh, buf, tmpu16) != tmpu16 ) {
   _CLOSE(fh);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  if ( muroar_read(fh, buf, 10) != 10 ) {
   _CLOSE(fh);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  if ( buf[1] != MUROAR_CMD_OK ) {
   _CLOSE(fh);
   _SET_ERRNO(EACCES);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  // Send AUTH command to server:
@@ -328,18 +360,18 @@ int muroar_connect(char * server, char * name) {
 
  if ( muroar_write(fh, buf, 10) != 10 ) {
   _CLOSE(fh);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  if ( muroar_read(fh, buf, 10) != 10 ) {
   _CLOSE(fh);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  if ( buf[1] != MUROAR_CMD_OK ) {
   _CLOSE(fh);
   _SET_ERRNO(EACCES);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  // We now have a working control connection, return it.
@@ -347,9 +379,9 @@ int muroar_connect(char * server, char * name) {
 }
 
 // Close a data connection
-int muroar_close(int fh) {
+int muroar_close(muroar_t fh) {
 
- if ( fh == -1 ) {
+ if ( fh == MUROAR_HANDLE_INVALID ) {
   _SET_ERRNO(EBADF);
   return -1;
  }
@@ -360,11 +392,11 @@ int muroar_close(int fh) {
 }
 
 // Close a control connection by sending QUIT command.
-int muroar_quit   (int fh) {
+int muroar_quit   (muroar_t fh) {
  char quit[] = "\0\6\0\0\0\0\0\0\0\0"; // QUIT command
  int ret = 0;
 
- if ( fh == -1 ) {
+ if ( fh == MUROAR_HANDLE_INVALID ) {
   _SET_ERRNO(EBADF);
   return -1;
  }
@@ -384,15 +416,15 @@ int muroar_quit   (int fh) {
 }
 
 // Open a Stream
-int muroar_stream(int fh, int dir, int * stream, int codec, int rate, int channels, int bits) {
+muroar_t muroar_stream(muroar_t fh, int dir, int * stream, int codec, int rate, int channels, int bits) {
  unsigned char buf[24];
  uint16_t sid;
  uint32_t * data = (uint32_t*)buf;
  int i;
 
- if ( fh == -1 ) {
+ if ( fh == MUROAR_HANDLE_INVALID ) {
   _SET_ERRNO(EBADF);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  // Send NEW_STREAM command:
@@ -404,7 +436,7 @@ int muroar_stream(int fh, int dir, int * stream, int codec, int rate, int channe
 
  if ( muroar_write(fh, buf, 10) != 10 ) {
   _CLOSE(fh);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  data[0] = dir;      // Stream Direction
@@ -419,18 +451,18 @@ int muroar_stream(int fh, int dir, int * stream, int codec, int rate, int channe
 
  if ( muroar_write(fh, buf, 24) != 24 ) {
   _CLOSE(fh);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  if ( muroar_read(fh, buf, 10) != 10 ) {
   _CLOSE(fh);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  if ( buf[1] != MUROAR_CMD_OK ) {
   _CLOSE(fh);
   _SET_ERRNO(EINVAL);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  // Stream ID of new stream is in byte 2 and 3 of header,
@@ -447,17 +479,17 @@ int muroar_stream(int fh, int dir, int * stream, int codec, int rate, int channe
 
  if ( muroar_write(fh, buf, 10) != 10 ) {
   _CLOSE(fh);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  if ( muroar_read(fh, buf, 10) != 10 ) {
   _CLOSE(fh);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  if ( buf[1] != MUROAR_CMD_OK ) {
   _CLOSE(fh);
-  return -1;
+  return MUROAR_HANDLE_INVALID;
  }
 
  // Set Stream ID in case caller want to know (passed non-NULL):
