@@ -31,10 +31,10 @@
 
 // Send notify beep.
 // We only support no-data default beeps.
-int muroar_beep   (int fh) {
+int muroar_beep   (muroar_t fh) {
  unsigned char beep[] = "\0\30\0\0\0\0\0\0\0\0"; // BEEP command
 
- if ( fh == -1 ) {
+ if ( fh == MUROAR_HANDLE_INVALID ) {
   _SET_ERRNO(EBADF);
   return -1;
  }
