@@ -21,13 +21,22 @@
  */
 
 #include "muroar.h"
-#include <unistd.h>
 
-int muroar_noop   (int fh) {
+// Set errno in case we have it
+#ifndef __WIN32
+#include <errno.h>
+#define _SET_ERRNO(x) (errno = (x))
+#else
+#define _SET_ERRNO(x)
+#endif
+
+int muroar_noop   (muroar_t fh) {
  unsigned char msg[10] = "\0\0\0\0\0\0\0\0\0\0";
 
- if ( fh == -1 )
+ if ( fh == MUROAR_HANDLE_INVALID ) {
+  _SET_ERRNO(EBADF);
   return -1;
+ }
 
  if ( muroar_write(fh, msg, 10) != 10 )
   return -1;
