@@ -21,8 +21,11 @@
  */
 
 #include "muroar.h"
-#include <stdio.h>
+#ifndef __WIN32
+#include <errno.h>
+#endif
 #include <unistd.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
@@ -84,7 +87,7 @@ void usage (char * progname) {
        );
 }
 
-void run_stream(int stream, int file, int tx) {
+void run_stream(muroar_t stream, int file, int tx) {
  char buf[1024];
  ssize_t len;
 
@@ -105,8 +108,12 @@ void run_stream(int stream, int file, int tx) {
  }
 }
 
+#ifdef __YIFF__
+yiff_uint_least16_t yiff_main(yiff_usize_t argc, char * argv[]) {
+#else
 int main (int argc, char *argv[]) {
- int fh;
+#endif
+ muroar_t fh;
  int filefh = -1;
  int dir    = MUROAR_PLAY_WAVE;
  int codec  = -1;     /* use default depending in dir */
@@ -114,9 +121,13 @@ int main (int argc, char *argv[]) {
  int chans  =  MUROAR_AID_CHANNELS;
  int bits   =  MUROAR_AID_BITS;
  int tx     =  1;
+#ifdef __YIFF__
+ yiff_usize_t i;
+#else
  int i;
+#endif
  char * k;
- char * server = NULL;
+ const char * server = NULL;
  const char * file   = NULL;
 
  for (i = 1; i < argc; i++) {
@@ -180,13 +191,22 @@ int main (int argc, char *argv[]) {
  }
 
  // Connect to server:
- if ( (fh = muroar_connect(server, NAME)) == -1 ) {
+ if ( (fh = muroar_connect(server, NAME)) == MUROAR_HANDLE_INVALID ) {
+#ifndef __WIN32
+  fprintf(stderr, "Error: Can not connect to server: %s: %s.\n",
+                  server == NULL ? "(default)" : server, strerror(errno));
+#else
   fprintf(stderr, "Error: Can not connect to server.\n");
+#endif
   return 1;
  }
 
- if ( muroar_stream(fh, dir, NULL, codec, rate, chans, bits) == -1 ) {
+ if ( (fh = muroar_stream(fh, dir, NULL, codec, rate, chans, bits)) == MUROAR_HANDLE_INVALID ) {
+#ifndef __WIN32
+  fprintf(stderr, "Error: Can not connect stream to server: %s\n", strerror(errno));
+#else
   fprintf(stderr, "Error: Can not connect stream to server.\n");
+#endif
   // We do not need to close fh here as it is already closed by muroar_stream().
   return 2;
  }
